@@ -22,7 +22,7 @@ from ..core.config import BaseConfig, AlertConfig, ZoneConfig
 @dataclass
 class AbandonedObjectConfig(BaseConfig):
     """Configuration for abandoned object detection use case."""
-    enable_smoothing: bool = True
+    enable_smoothing: bool = False
     centroid_threshold: float = 10.0
     smoothing_algorithm: str = "observability"
     smoothing_window_size: int = 20
