@@ -127,8 +127,8 @@ class ColorDetectionUseCase(BaseProcessor):
         self.smoothing_tracker = None  # BBoxSmoothingTracker instance
         self._total_frame_counter = 0  # Total frames processed
         self._global_frame_offset = 0  # Frame offset for new sessions
-        self._color_total_track_ids = {}  # Cumulative track IDs per category and color
-        self._color_current_frame_track_ids = {}  # Per-frame track IDs per category and color
+        self._color_total_track_ids = defaultdict(set)  # Cumulative track IDs per category-color
+        self._color_current_frame_track_ids = defaultdict(set)  # Per-frame track IDs per category-color
         
         self._tracking_start_time = None
         
@@ -149,8 +149,8 @@ class ColorDetectionUseCase(BaseProcessor):
 
     def reset_color_tracking(self) -> None:
         """Reset color tracking state."""
-        self._color_total_track_ids = {}
-        self._color_current_frame_track_ids = {}
+        self._color_total_track_ids = defaultdict(set)
+        self._color_current_frame_track_ids = defaultdict(set)
         self._total_frame_counter = 0
         self._global_frame_offset = 0
         self.logger.info("Color tracking state reset")
@@ -249,7 +249,13 @@ class ColorDetectionUseCase(BaseProcessor):
     
     def _update_color_tracking_state(self, detections: List[Dict]):
         """Track unique track_ids per category and color for total count."""
-        self._color_total_track_ids = getattr(self, '_color_total_track_ids', defaultdict(set))
+        # Ensure storage is a defaultdict(set) to allow safe .add()
+        existing_store = getattr(self, '_color_total_track_ids', None)
+        if not isinstance(existing_store, defaultdict):
+            existing_store = {} if existing_store is None else dict(existing_store)
+            self._color_total_track_ids = defaultdict(set, existing_store)
+        else:
+            self._color_total_track_ids = existing_store
         self._color_current_frame_track_ids = defaultdict(set)
         for det in detections:
             cat = det.get('category')
@@ -261,8 +267,20 @@ class ColorDetectionUseCase(BaseProcessor):
                 self._color_current_frame_track_ids[key].add(track_id)
 
     def get_total_color_counts(self):
-        """Return total unique track_id count for each category-color pair."""
-        return {key: len(ids) for key, ids in getattr(self, '_color_total_track_ids', {}).items()}
+        """Return total unique track_id count per category (across all colors)."""
+        store = getattr(self, '_color_total_track_ids', {})
+        if not isinstance(store, dict):
+            return {}
+        category_to_ids = defaultdict(set)
+        for key, id_set in store.items():
+            if isinstance(key, str) and ':' in key:
+                cat, _ = key.split(':', 1)
+            else:
+                cat = key
+            # Support both set and iterable
+            ids = id_set if isinstance(id_set, set) else set(id_set or [])
+            category_to_ids[cat].update(ids)
+        return {cat: len(ids) for cat, ids in category_to_ids.items()}
 
     def _get_track_ids_info(self, detections: List[Dict]) -> Dict[str, Any]:
         """Get detailed information about track IDs for color detections (per frame)."""
@@ -335,6 +353,9 @@ class ColorDetectionUseCase(BaseProcessor):
             if config.target_categories:
                 color_processed_data = [d for d in processed_data if d.get('category') in self.target_categories]
                 self.logger.debug("Applied category filtering")
+                print("-------------------COLOR_PROCESSED_DATA-------------------")
+                print(color_processed_data)
+                print("-------------------COLOR_PROCESSED_DATA-------------------")
             
             # Step 2.5: Filter to only include target categories
             # color_processed_data = filter_by_categories(processed_data.copy(), config.target_categories)
@@ -393,6 +414,9 @@ class ColorDetectionUseCase(BaseProcessor):
                 input_bytes, 
                 config
             )
+            print("-------------------COLOR_ANALYSIS-------------------")
+            print(color_analysis)
+            print("-------------------COLOR_ANALYSIS-------------------")
             
             # Step 8: Calculate summaries
             color_summary = self._calculate_color_summary(color_analysis, config)
@@ -683,28 +707,44 @@ class ColorDetectionUseCase(BaseProcessor):
                 "frame_id": record["frame_id"],
                 "main_color": record["main_color"]
             })
+        print("-------------------category_colors-------------------")
+        print(category_colors)
+        print("-------------------category_colors-------------------")
             
         self.logger.debug(f"Valid detections after filtering: {len(detections)}")
         summary = {
             "total_count": sum(counts.values()),
             "per_category_count": counts,
-            "detections": detections
+            "detections": detections,
+            "dominant_colors": {},
         }
+        print("-------------------summary-------------------")
+        print(summary)
+        print("-------------------summary-------------------")
 
         all_colors = defaultdict(int)
         for category_data in category_colors.values():
             for color, count in category_data.items():
                 all_colors[color] += count
         summary["color_distribution"] = dict(all_colors)
+        print("-------------------summary1-------------------")
+        print(summary)
+        print("-------------------summary1-------------------")
 
         for category, colors in category_colors.items():
             if colors:
-                dominant_color = max(colors.items(), key=lambda x: x[1])
-                summary["dominant_colors"][category] = {
-                    "color": dominant_color[0],
-                    "count": dominant_color[1],
-                    "percentage": round((dominant_color[1] / sum(colors.values())) * 100, 1)
-                }
+                if "dominant_colors" not in summary:
+                    summary["dominant_colors"] = {}
+                else:
+                    dominant_color = max(colors.items(), key=lambda x: x[1])
+                    summary["dominant_colors"][category] = {
+                        "color": dominant_color[0],
+                        "count": dominant_color[1],
+                        "percentage": round((dominant_color[1] / sum(colors.values())) * 100, 1)
+                    }
+        print("-------------------summary2-------------------")
+        print(summary)
+        print("-------------------summary2-------------------")
 
         return summary
         
@@ -905,7 +945,7 @@ class ColorDetectionUseCase(BaseProcessor):
         # frame_tracking_stats = tracking_stats[0][frame_key]
         tracking_stats = []
 
-        total_detections = counting_summary.get("total_detections", 0)
+        total_detections = counting_summary.get("total_count", 0)
         total_counts = counting_summary.get("total_color_counts", {})
         cumulative_total = sum(total_counts.values()) if total_counts else 0
         per_category_count = counting_summary.get("per_category_count", {})
@@ -1127,7 +1167,7 @@ class ColorDetectionUseCase(BaseProcessor):
         # Use frame number as key, fallback to 'current_frame' if not available
         frame_key = str(frame_number) if frame_number is not None else "current_frame"
         incidents=[]
-        total_detections = counting_summary.get("total_detections", 0)
+        total_detections = counting_summary.get("total_count", 0)
         current_timestamp = self._get_current_timestamp_str(stream_info)
         camera_info = self.get_camera_info_from_stream(stream_info)
         
@@ -1235,15 +1275,17 @@ class ColorDetectionUseCase(BaseProcessor):
 
         frame_key = str(frame_number) if frame_number is not None else "current_frame"
         alerts = []
-        total_detections = summary.get("total_detections", 0) #CURRENT combined total count of all classes
+        total_detections = summary.get("total_count", 0) #CURRENT combined total count of all classes
         total_counts_dict = summary.get("total_color_counts", {}) #TOTAL cumulative counts per class
+        if isinstance(total_counts_dict, int):
+            total_counts_dict = {}
         cumulative_total = sum(total_counts_dict.values()) if total_counts_dict else 0 #TOTAL combined cumulative count
         per_category_count = summary.get("per_category_count", {}) #CURRENT count per class
 
         if not config.alert_config:
             return alerts
 
-        total = summary.get("total_detections", 0)
+        total = summary.get("total_count", 0)
         #self._ascending_alert_list
         if hasattr(config.alert_config, 'count_thresholds') and config.alert_config.count_thresholds:
 
